<?php
/**
* Linked glossary entries can't be output in
* PDF. This script creates a custom glossary page for PDF
* output. All glossary topic files must be in the same directory
* and this script must be executed from that directory.
* The styles for div.gloss_term and
* div.gloss_letter are in the page <head>.
* © 2017 Moneris Solutions Corporation, CC-BY-NC -SA 4.0
* @param string $argv[1] The path to your flglo file
*/
$source = $argv[1];
//save to current directory
$dest = "pdf_glossary.html";
$doc = new DOMDocument();
$doc->formatOutput = true;
$doc->load($source);
$xpath = new DOMXPath($doc);
//find all glossary entries
$nodes = $xpath->query("//GlossaryEntry");
$arr = Array();
foreach ($nodes as $node){
  $value = getDefinition($node);
  $term = getNodeValues($node);
  $arr[$term] = $value;
}
ksort($arr, SORT_NATURAL | SORT_FLAG_CASE);
$new_doc = createPage($arr);
//Create in the directory where
//the glossary topic files are.
//This way any hrefs will be valid.
$new_doc->save($dest);
//end main
/**
* Iterate over node and return
* definition or link
* @param DOMNode $node Node containing the definition
* @return string
*/
function getDefinition($node){
  $definition = "";
  foreach($node->childNodes as $n) {
    if($n->nodeName == "Definition"){
      // link or text
      if(trim($n->nodeValue) != ""){
        $definition = trim($n->nodeValue);
      } else {
        $definition = basename($n->getAttribute("Link"));
      }
    }
  }
  return $definition;
}
/**
* There may be more than one Term
* so get all the terms.
* @param $node DOMNode The node containing the terms
* @return string
*/
function getNodeValues($node){
  $value = "";
  foreach ($node->childNodes as $n) {
    if($n->nodeName=="Terms"){
      foreach($n->childNodes as $t){
        if ($t->nodeName == "Term"){
          $value .= trim($t->nodeValue) .", ";
        }
      }
    }
  }
  return trim($value, ", ");
}
/**
* Create new Flare glossary page
* with styles embedded in the head.
* @param Array $a Terms and their matching values
*/
function createPage($a){
  $new_glossary = "<?xml version=\"1.0\" encoding=\"utf-8\"?>
  <html xmlns=\"http://www.w3.org/1999/xhtml\"
  xmlns:MadCap=\"http://www.madcapsoftware.com/Schemas/MadCap.xsd\"
  MadCap:conditions=\"General.Print\">
    <head>
    <title>Glossary</title>
      <style type=\"text/css\">
        div.gloss_letter
        {
          padding: .25em;
          margin-top: .5em;
          margin-bottom: .75em;
          font-size: 16pt;
          background-color: #ddd;
          page-break-after: avoid;
        }
        div.gloss_term
        {
          font-weight: bold;
          page-break-after: avoid;
        }
          h1.no_number
        {
          mc-auto-number-format: '';
        }
      </style>
    </head>
  </html>
  ";
  //create new DOMDocument
  $new_doc = new DOMDocument();
  $new_doc->loadXML($new_glossary);
  $ns = "http://www.w3.org/1999/xhtml";
  $body_node = $new_doc->createElementNS($ns,'body');
  $new_doc->documentElement->appendChild($body_node);
  $header = $new_doc->createElement('h1', "Glossary");
  $header->setAttribute("class", "no_number");
  $body_node->appendChild($header);
  //keep track of previous letter to determine
  //when to start a new header
  $previous_letter = "";
  foreach ($a as $key => $value) {
    $first_letter = strtolower(substr($key, 0, 1));
    if($first_letter != $previous_letter){
      //create new div
      $letter_div = $new_doc->createElement('div',
      strtoupper($first_letter));
      //set class attribute
      $letter_div->setAttribute("class", "gloss_letter");
      //append to body
      $body_node->appendChild($letter_div);
      $previous_letter = strtolower($first_letter);
    }
    $term_div = $new_doc->createElement('div', $key);
    $term_div->setAttribute("class", "gloss_term");
    $body_node->appendChild($term_div);
    //now load file and add to the body
    termDescription($new_doc, $body_node, $value);
  }
  return $new_doc;
}
/**
* Build the Description of each term.
* @param DOMDocument $new_doc The new glossary
* @param DOMNode $body_node The body node of the new glossary
* @param string $value The glossary topic URL or the plain text
*/
function termDescription($new_doc, $body_node, $value){
  //check if value is a file
  if (file_exists($value)){
    $d = new DOMDocument();
    $d->formatOutput = true;
    $d->load($value);
    $xp = new DOMXPath($d);
    $ns = $d->documentElement->namespaceURI;
    if($ns) {
     $xp->registerNamespace("ns", $ns);
     $nodes = $xp->query("//ns:body");
    } else {
      $nodes = $xp->query("//body");
    }
    foreach ($nodes as $node){
      foreach ($node->childNodes as $n) {
        if ( !preg_match('/h[1-3]/', $node->nodeName)) {
          $import = $new_doc->importNode($node, true);
          $body_node->appendChild($import);
        }
      }
    }
  } else { //plain text, create a <p>
    $pnode = $new_doc->createElement('p', $value);
    $body_node->appendChild($pnode);
  }
}
?>
