<?php
/**
* Linked glossary entries can't be output in PDF.
* This script creates a custom glossary page for PDF
* output. All glossary topic files must be in the same
* directory and this script must be executed from that
* directory. The styles for p.gloss_term and 
* p.gloss_letter are in the page <head>.
* © 2017 Moneris Solutions Corporation, CC-BY-NC -SA 4.0
* @param string $argv[1] The path to your flglo file
*/
// flglo file
$source = $argv[1]; 
//save to current directory
$dest = "pdf_glossary.html";
$doc = new DOMDocument();
$doc->formatOutput = true;
$doc->load($source);
$xpath = new DOMXPath($doc);
//get all GlossaryEntry nodes
$nodes = $xpath->query("//GlossaryEntry");
//create array of sort value and unique ID
$arr = Array();
foreach ($nodes as $node){ 
  $sort = getSortAs($node);
  $arr[$sort] = $node->getAttribute("glossTerm");
}
//order by sort value
ksort($arr, SORT_NATURAL | SORT_FLAG_CASE); 
$new_doc = createPage($arr, $doc); 
//save file
$new_doc->save($dest);
//end main
/**
* Sort by term or SortAs
* @param $node DOMNode The node containing the terms
* @return string
*/
function getSortAs($node){ 
  if ($node->hasAttribute("SortAs") && 
    $node->getAttribute("SortAs") != ""){
    return $node->getAttribute("SortAs");
  }else { //get Term
    $term = getNodeValues($node);
  }
  return $term; 
}
/**
* There may be more than one Term
* so get all the terms.
* @param $node DOMNode The node containing the terms
* @return string of all Terms
*/
function getNodeValues($node){ 
  $value = "";
  foreach ($node->childNodes as $n) {
    if($n->nodeName=="Terms"){
      foreach($n->childNodes as $t){
        if ($t->nodeName == "Term"){
          $value .= trim($t->nodeValue) .", ";
        }
      }
    }      
  }
  return trim($value, ", ");
}
/**
* Create new Flare glossary page
* with styles embedded in the head.
* @param Array       $a   glossTerm (value), Term or SortAs (key)
* @param DOMDocument $doc The Flare glossary
*/
function createPage($a, $doc){
  $new_glossary = "<?xml version=\"1.0\" encoding=\"utf-8\"?>
  <html xmlns=\"http://www.w3.org/1999/xhtml\"
    xmlns:MadCap=\"http://www.madcapsoftware.com/Schemas/MadCap.xsd\"
    MadCap:conditions=\"Default.PrintOnly\"> 
    <head>
    <title>Glossary</title>
    <style type=\"text/css\"> 
      p.gloss_letter
      {
        padding: .25em;
        margin-top: .5em;
        margin-bottom: .75em;
        font-size: 16pt;
        background-color: #ddd;
        page-break-after: avoid;
      }
      p.gloss_term
      {
        font-weight: bold;
        page-break-after: avoid;
      }
      h1.no_number
      {
        mc-auto-number-format: ''; 
      }
    </style>
    </head> 
  </html>
  ";
  //create new DOMDocument for HTML page
  $new_doc = new DOMDocument();
  $new_doc->loadXML($new_glossary);
  $ns = "http://www.w3.org/1999/xhtml";
  $body_node = $new_doc->createElementNS($ns,'body'); 
  $new_doc->documentElement->appendChild($body_node);
  $header = $new_doc->createElement('h1', "Glossary");
  $header->setAttribute("class", "no_number");
  $body_node->appendChild($header);
  //keep track of previous letter to determine
  //when to start a new header
  $previous_letter = "";
  foreach ($a as $key => $value) {
    //look up term & definition based on glossTerm (value)
    $glossary_entry_node = getGlossaryEntry($value, $doc);
    $term = getNodeValues($glossary_entry_node);
    $def = getDefinition($glossary_entry_node);
    //get first letter from SortAs/Term 
    $first_letter = strtolower(substr($key, 0, 1));
    if($first_letter != $previous_letter){
      //create new p 
      $letter_p = $new_doc->createElement('p',
        strtoupper($first_letter));
      //set class attribute
      $letter_p->setAttribute("class", "gloss_letter");
      //append to body
      $body_node->appendChild($letter_p);
      $previous_letter = strtolower($first_letter);
    }
    $term_p = $new_doc->createElement('p', $term);
    $term_p->setAttribute("class", "gloss_term"); 
    $body_node->appendChild($term_p);
    //now load file and add to the body
    termDescription($new_doc, $body_node, $def);
  }
  return $new_doc;
}
/**
* Use the glossTerm to find Term and Definition
* @param string      $glossTerm The glossTerm
* @param DOMDocument $doc       DOM of the Flare glossary
* @return DOMNode associated with glossTerm
*/
function getGlossaryEntry($glossTerm, $doc){ 
  $xpath = new DOMXPath($doc);
  $nodes = $xpath->query("//GlossaryEntry[@glossTerm='" . $glossTerm . "']");
  //only one node
  $n = $nodes->item(0);  
  return $n;
}
/**
* Iterate over node and return
* definition or link
* @param DOMNode $node Node containing the definition
* @return string (either link or text definition)
*/
function getDefinition($node){ 
  $definition = "";
  foreach($node->childNodes as $n) {    
    if($n->nodeName == "Definition"){
      // link or text
      if(trim($n->nodeValue) != ""){
        $definition = trim($n->nodeValue);
      } else {
        $definition = basename($n->getAttribute("Link"));
      }
    }
  }
  return $definition;
}
/**
* Build the Description of each term
* The $value parameter has the file name
* or the simple text value.
* @param DOMDocument $new_doc   The new HTML document
* @param DOMNode     $body_node Append the new term to body
* @param string      $value     The definition (or link to topic)
*/
function termDescription($new_doc, $body_node, $value){
  //check if value is a file
  if (file_exists($value)){
    $d = new DOMDocument();
    $d->formatOutput = true;
    $d->load($value);
    $xp = new DOMXPath($d);
    $ns = $d->documentElement->namespaceURI;
    if($ns) {
      $xp->registerNamespace("ns", $ns);
      $nodes = $xp->query("//ns:body");
    } else {
      $nodes = $xp->query("//body");
    }
    //get all children of body except header
    foreach ($nodes->item(0)->childNodes as $node){
      if ( !preg_match('/h[1-3]/', $node->nodeName)) {  
        $import = $new_doc->importNode($node, true); 
        $body_node->appendChild($import);  
      }    
    }
  } else { //plain text, create a <p> 
    $pnode = $new_doc->createElement('p', $value);
    $body_node->appendChild($pnode);  
  }
}
?>